"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useHasHydrated = exports.useIsHydrating = exports.useMarkAsHydrated = exports.useBoundaryHydrater = exports.HydrationProvider = void 0;
const tslib_1 = require("tslib");
const react_1 = (0, tslib_1.__importStar)(require("react"));
const retil_boundary_1 = require("retil-boundary");
const retil_mount_1 = require("retil-mount");
const retil_source_1 = require("retil-source");
const hydrationEnvService_1 = require("./hydrationEnvService");
const OnHydrationDetectedContext = /*#__PURE__*/ (0, react_1.createContext)(null);
const HydratingContext = /*#__PURE__*/ (0, react_1.createContext)(undefined);
/**
 * An optional provider that allows for manual control of the hydration state.
 *
 * When not provided, the hydration hooks will instead create and interact with
 * a default global hydration service.
 */
const HydrationProvider = (props) => {
    const { children, onHydrationDetected, isHydrating } = props;
    return (react_1.default.createElement(HydratingContext.Provider, { value: isHydrating },
        react_1.default.createElement(OnHydrationDetectedContext.Provider, { value: onHydrationDetected }, children)));
};
exports.HydrationProvider = HydrationProvider;
/**
 * Sets up retil-boundary based hydration detection for your app.
 *
 * This should be called once at the top level of your app, above any
 * <Boundary> components. It'll then caue `useIsHydrating()` to become `false`
 * after the first render of the deepest <Boundary> component.
 */
function useBoundaryHydrater() {
    const markAsHydrated = useMarkAsHydrated();
    (0, retil_boundary_1.useBoundaryEffect)(markAsHydrated);
}
exports.useBoundaryHydrater = useBoundaryHydrater;
/**
 * Returns a function that can be called to manually mark the app as hydrated.
 *
 * When used in conjunction with a `<HydrationProvider>`, this will call the
 * provided `onHydrationDetected` function. Otherwise, it'll call the default
 * hydration service's `hydrate` method.
 */
function useMarkAsHydrated() {
    const contextOnHydrationDetected = (0, react_1.useContext)(OnHydrationDetectedContext);
    return contextOnHydrationDetected || (0, hydrationEnvService_1.getDefaultHydrationEnvService)()[1];
}
exports.useMarkAsHydrated = useMarkAsHydrated;
/**
 * Pulls the latest hydration state from the first available source out of:
 *
 * - React Context (as set by a <HydrationProvider>)
 * - retil-mount's useEnv hook, if hydration state is set on the env
 * - the default hydration env service
 *
 *
 */
function useIsHydrating() {
    const env = (0, retil_mount_1.useMountEnv)();
    const contextHydrating = (0, react_1.useContext)(HydratingContext);
    const source = (0, hydrationEnvService_1.getDefaultHydrationEnvService)()[0];
    const serviceHydrating = (0, retil_source_1.useSource)(env ? null : source, {
        defaultValue: env,
    });
    return contextHydrating !== null && contextHydrating !== void 0 ? contextHydrating : serviceHydrating === null || serviceHydrating === void 0 ? void 0 : serviceHydrating.hydrating;
}
exports.useIsHydrating = useIsHydrating;
function useHasHydrated() {
    return useIsHydrating() === false;
}
exports.useHasHydrated = useHasHydrated;
